############################################
# Ivan Marti-Vidal (2020), Univ. Valencia. #
############################################

# Generate a synthetic wideband EVN experiment with PolSimulate.
# Then, tclean in mfs mode (with nterms=1 and 2).


import numpy as np
import os


DOSIM = True ## REDO POLSIMULATE
DOMFS1 = True ## REDO MFS WITH NTERMS=1
DOMFS2 = True ## REDO MFS WITH NTERMS=2

##############################################
### Run PolSimulate:

if DOSIM:
 polsimulate(
  vis                =  "polsimulate_WideBand.ms", # output ms name.
  reuse              =  False,           # Re-generate it, if it already exists.
  array_configuration    =  "EVN.cfg",    # Antenna locations.
  elevation_cutoff    =  5.0,             # Flag elevations lower than this.
  feed               =  "circular",       # Polarization feeds.
  mounts             =  [],               # Mount type for each antenna (alt-az, by default)
  ConstDt0           =  [],               # Dterms for each antenna in first pol. channel (0, by default).
  ConstDt1           =  [],               # Dterms for the second pol. channel (0, by default)
  LO                 =  5000000000.0,     # Local Oscillator (i.e., frequency of observation).
  BBs                =  [0.0],            # Baseband frequency offset(s). We only simulate one spw.
  spw_width          =  512000000.0,     # Width of the spectral window.
  nchan              =  8,                # Number of channels (per spw).
  model_image        =  [],               # CASA model image (not required).
  I                  =  [1.0, 0.5],       # Fluxes of point sources to simulate (just two deltas).
  Qfrac              =  [0.0, 0.0],       # Fractional Stokes Q of each delta.
  Ufrac              =  [0.0, 0.0],       # Fractional Stokes U of each delta.
  Vfrac              =  [0.0, 0.0],       # Fractional Stokes V of each delta.
  RM                 =  [0.0, 0.0],       # Rotation Measure of each delta.
  spec_index         =  [0.0, -2.0],      # Spectral index of each delta (i.e., thick core; thin component).
  RAoffset           =  [0.0, -2.5/3.6e6],  # RA offset of each delta (in degrees; e.g., centered core and 2.5 mas offset component).
  Decoffset          =  [0.0, 2.5/3.6e6],   # Dec offset of each delta.
  spectrum_file      =  "",                 # ascii file with the spectrum of each delta (for line simulations).
  phase_center       =  "J2000 12h00m00.08s 30d00m09.0s",  
  incell             =  "",     # Not used (i.e., no CASA image is being used in the simulation).
  inbright           =  0.0,    # Not used.
  inwidth            =  "",     # Not used.
  innu0              =  "",     # Not used.
  H0                 =  -6.0,   # Starting hour angle at array center (for VLBI, GMT).
  onsource_time      =  2.0,    # Total time on source (i.e., including overheads).
  observe_time       =  12.0,   # Total observing time.
  visib_time         =  "30s",  # Integration time at the correlator (pretty large, to save disk space).
  nscan              =  50,     # Number of scans (equally spaced).
  apply_parang       =  True,   # Apply parallactic-angle correction to the data.
  export_uvf         =  True,   # Generate uvfits file as well.
  corrupt            =  True,   # Corrupt the data with thermal noise.
  seed               =  42,     # Seed of the random number generator.
  Dt_amp             =  0.0,    # Mean amplitude for Dterm spectral noise.
  Dt_noise           =  0.001,  # Std for Dterm spectral noise.
  tau0               =  0.0,    # Atmospheric opacity (the same for all antennas).
  t_sky              =  250.0,  # Sky temperature.
  t_ground           =  270.0,  # Ground temperature.
  t_receiver         =  50.0   # Receiver temperature (the same for all antennas).
 )


clearcal(vis="polsimulate_WideBand.ms", addmodel=True)
MASK = ['box[[1020pix,1014pix],[1028pix,1034pix]]','box[[1045pix,1039pix],[1054pix,1058pix]]']


## Create the mask file:
OFF = open('EVN_MFS.msk','w')
print >> OFF, '#CRTF\n'
for msk in MASK:
  print >> OFF,msk+'\n'
OFF.close()


#####################
## TCLEAN 1:
if DOMFS1:
 os.system('rm -rf SPECTRUM_MFS1.*')
 tclean(
  vis                =  "polsimulate_WideBand.ms",
  imagename          =  "SPECTRUM_MFS1",
  imsize             =  2048,
  cell               =  "0.0001arcsec",
  specmode           =  "mfs",
  gridder            =  "standard",
  deconvolver        =  "mtmfs",
  scales             =  [],
  nterms             =  1,
  restoration        =  True,
  weighting          =  "natural",
  niter              =  1000,
  gain               =  0.1,
  cycleniter         =  -1,
  interactive        =  False,
  usemask            =  "user",
  mask               =  'EVN_MFS.msk',
  savemodel          =  "none",
  calcres            =  True,
  calcpsf            =  True,
  parallel           =  False
 )



#####################
## TCLEAN 2:
if DOMFS2:
 os.system('rm -rf SPECTRUM_MFS2.*')
 tclean(
  vis                =  "polsimulate_WideBand.ms",
  imagename          =  "SPECTRUM_MFS2",
  imsize             =  2048,
  cell               =  "0.0001arcsec",
  specmode           =  "mfs",
  gridder            =  "standard",
  deconvolver        =  "mtmfs",
  scales             =  [],
  nterms             =  2,
  restoration        =  True,
  weighting          =  "natural",
  niter              =  1000,
  gain               =  0.1,
  cycleniter         =  -1,
  interactive        =  False,
  usemask            =  "user",
  mask               =  'EVN_MFS.msk',
  savemodel          =  "none",
  calcres            =  True,
  calcpsf            =  True,
  parallel           =  False
 )





#############
## GENERATE PLOT:
## Load the *.rstr file in imview.


